@@ -53,7 +53,7 @@ static void snd_async_handler(int signo ATTRIBUTE_UNUSED, siginfo_t *siginfo, vo
 {
 	int fd;
 	struct list_head *i;
-	assert(siginfo->si_code = SI_SIGIO);
+	//assert(siginfo->si_code == SI_SIGIO);
 	fd = siginfo->si_fd;
 	list_for_each(i, &snd_async_handlers) {
 		snd_async_handler_t *h = list_entry(i, snd_async_handler_t, glist);
@@ -151,9 +151,11 @@ int snd_async_del_handler(snd_async_handler_t *handler)
 	if (!list_empty(&handler->hlist))
 		goto _end;
 	switch (handler->type) {
+#ifdef BUILD_PCM
 	case SND_ASYNC_HANDLER_PCM:
 		err = snd_pcm_async(handler->u.pcm, -1, 1);
 		break;
+#endif
 	case SND_ASYNC_HANDLER_CTL:
 		err = snd_ctl_async(handler->u.ctl, -1, 1);
 		break;
