@@ -197,10 +197,11 @@ static char *get_dev_name(struct hint_list *list)
 			free(str1);
 			return res;
 		}
-	} else {
-		return strdup(list->cardname);
 	}
-	return NULL;
+	/* if the specified device doesn't exist, skip this entry */
+	if (list->device_input >= 0 || list->device_output >= 0)
+		return NULL;
+	return strdup(list->cardname);
 }
 
 #ifndef DOC_HIDDEN
@@ -304,8 +305,8 @@ static int try_config(struct hint_list *list,
 				err = -EINVAL;
 				goto __cleanup;
 			}
-			list->device_input = -1;
-			list->device_output = -1;
+			list->device_input = dev;
+			list->device_output = dev;
 		}
 		if (snd_config_search(cfg, "device_input", &n) >= 0) {
 			if (snd_config_get_integer(n, &list->device_input) < 0) {
