@@ -53,6 +53,10 @@ typedef struct {
 	snd_seq_event_t out_event;
 	int pending;
 } snd_rawmidi_virtual_t;
+
+int _snd_seq_open_lconf(snd_seq_t **seqp, const char *name, 
+			int streams, int mode, snd_config_t *lconf,
+			snd_config_t *parent_conf);
 #endif
 
 static int snd_rawmidi_virtual_close(snd_rawmidi_t *rmidi)
@@ -85,9 +89,9 @@ static int snd_rawmidi_virtual_info(snd_rawmidi_t *rmidi, snd_rawmidi_info_t * i
 	info->device = 0;
 	info->subdevice = 0;
 	info->flags = 0;
-	strcpy(info->id, "Virtual");
-	strcpy(info->name, "Virtual RawMIDI");
-	strcpy(info->subname, "Virtual RawMIDI");
+	strcpy((char *)info->id, "Virtual");
+	strcpy((char *)info->name, "Virtual RawMIDI");
+	strcpy((char *)info->subname, "Virtual RawMIDI");
 	info->subdevices_count = 1;
 	info->subdevices_avail = 0;
 	return 0;
@@ -250,7 +254,7 @@ static ssize_t snd_rawmidi_virtual_read(snd_rawmidi_t *rmidi, void *buffer, size
 			} else {
 				virt->in_buf_ptr = virt->in_tmp_buf;
 				virt->in_buf_size = snd_midi_event_decode(virt->midi_event,
-									  virt->in_tmp_buf,
+									  (unsigned char *)virt->in_tmp_buf,
 									  sizeof(virt->in_tmp_buf),
 									  virt->in_event);
 			}
@@ -381,11 +385,10 @@ int snd_rawmidi_virtual_open(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp,
 		snd_seq_close(seq_handle);
 	if (virt->midi_event)
 		snd_midi_event_free(virt->midi_event);
-	if (virt)
-		free(virt);
-	if (inputp && *inputp)
+	free(virt);
+	if (inputp)
 		free(*inputp);
-	if (outputp && *outputp)
+	if (outputp)
 		free(*outputp);
 	return err;
 }
@@ -408,9 +411,7 @@ int _snd_rawmidi_virtual_open(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp,
 		const char *id;
 		if (snd_config_get_id(n, &id) < 0)
 			continue;
-		if (strcmp(id, "comment") == 0)
-			continue;
-		if (strcmp(id, "type") == 0)
+		if (snd_rawmidi_conf_generic_id(id))
 			continue;
 		if (strcmp(id, "slave") == 0) {
 			err = snd_config_get_string(n, &slave_str);
@@ -435,11 +436,12 @@ int _snd_rawmidi_virtual_open(snd_rawmidi_t **inputp, snd_rawmidi_t **outputp,
 
 	seq_mode = 0;
 	if (mode & SND_RAWMIDI_NONBLOCK)
-		seq_mode |= O_NONBLOCK;
+		seq_mode |= SND_SEQ_NONBLOCK;
 
 	if (! slave_str)
 		slave_str = "default";
-	err = snd_seq_open_lconf(&seq_handle, slave_str, streams, seq_mode, root);
+	err = _snd_seq_open_lconf(&seq_handle, slave_str, streams, seq_mode,
+				  root, conf);
 	if (err < 0)
 		return err;
 
